package com.tanpu.fund.service.impl;

import com.tanpu.common.model.product.resp.Net;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import com.github.pagehelper.page.PageMethod;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.tanpu.common.auth.UserInfoThreadLocalHolder;
import com.tanpu.common.enums.BizEnums;
import com.tanpu.common.enums.Constant;
import com.tanpu.common.enums.SysConstEnums;
import com.tanpu.common.enums.product.ProductEnums;
import com.tanpu.common.model.Page;
import com.tanpu.common.model.Pageable;
import com.tanpu.common.model.adm.req.IfaNetImportReq;
import com.tanpu.common.model.fund.req.FundRankReq;
import com.tanpu.common.model.fund.resq.FundRankResp;
import com.tanpu.common.model.privatefund.resp.FundNavMessage;
import com.tanpu.common.model.product.FundInfoVO;
import com.tanpu.common.model.product.req.NetReq;
import com.tanpu.common.model.product.req.ProductInfoReq;
import com.tanpu.common.model.product.req.ProductListReq;
import com.tanpu.common.model.product.resp.*;
import com.tanpu.common.model.tanpuroom.Type;
import com.tanpu.common.model.user.resp.SysConstantResp;
import com.tanpu.common.resp.CommonResp;
import com.tanpu.common.utils.BigDecimalUtil;
import com.tanpu.fund.entity.generator.*;
import com.tanpu.fund.enums.FilterTypeEnum;
import com.tanpu.fund.feign.diagnose.FeignClientForDiagnose;
import com.tanpu.fund.feign.product.FeignForProduct;
import com.tanpu.fund.feign.user.FeignClientForFatools;
import com.tanpu.fund.mapper.generator.*;
import com.tanpu.fund.mapper.generator.custom.FundInfoCustomMapper;
import com.tanpu.fund.service.ProductService;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.validator.routines.BigDecimalValidator;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import static com.tanpu.common.utils.BigDecimalUtil.multiply100;
import static com.tanpu.common.utils.BigDecimalUtil.subtract;
import static com.tanpu.common.utils.BigDecimalUtil.subtractAbs;
import static com.tanpu.fund.enums.FilterTypeEnum.SINCE_ESTABLISHED;
import static com.tanpu.fund.utils.LongUtil.timeLong;

/**
 * @author: zhoupeng
 * @email: zhoupeng_08@163.com
 */
@Service
public class ProductServiceImpl implements ProductService, Constant {

    @Value("${file.preview.path}")
    private String suffix;

    @Value("${company.oss.prefix}")
    private String filePrefix;

    @Resource
    private FundInfoMapper fundInfoMapper;

    @Resource
    private FundPerformanceMapper fundPerformanceMapper;

    @Resource
    private FundNavMapper fundNavMapper;

    @Resource
    private FundManagerMappingMapper fundManagerMappingMapper;

    @Resource
    private PersonnelInfoMapper personnelInfoMapper;

    @Resource
    private CompanyInfoMapper companyInfoMapper;

    @Resource
    private FundCountMapper fundCountMapper;

    @Resource
    private IfaImportedFundCountMapper ifaImportedFundCountMapper;

    @Resource
    private FundInfoCustomMapper fundInfoCustomMapper;

    @Resource
    private FundRiskStatsMapper fundRiskStatsMapper;

    @Resource
    private FundRateMappingMapper fundRateMappingMapper;

    @Resource
    private FundDistributionMapper fundDistributionMapper;

    @Resource
    private IfaImportedFundInfoMapper ifaImportedFundInfoMapper;
    @Resource
    private IfaImportedFundNavMapper ifaImportedFundNavMapper;

    @Resource
    private FeignClientForFatools fatools;

    @Resource
    private FeignClientForDiagnose feignClientForDiagnose;

    @Resource
    private FeignForProduct product;

    @Override
    public Page<ProductInfoVO> getProductList(ProductInfoReq req) {

        // 策略集合
        com.github.pagehelper.Page<ProductInfoVO> page = PageMethod.startPage(req.getPage().getPageNumber(),
                req.getPage().getPageSize());
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(reqFilterResult(req));

        List<String> list = fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList());

        List<ProductInfoVO> vos = new ArrayList<>();

        if (CollectionUtils.isNotEmpty(list)) {
            vos = getProductInfoVOS(fundCounts, list);
        }

        return new Page<>(req.getPage(), page.getTotal(), vos);
    }

    @Override
    public ProductInfoVO getProductDetail(String id) {

        ProductInfoVO vo = new ProductInfoVO();
        vo.setFundId(id);
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("data_sources");

        List<FundInfo> infos = this.fundInfoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(infos)) {
            return new ProductInfoVO();
        }
        FundInfo fundInfo = infos.get(0);
        BeanUtils.copyProperties(fundInfo, vo);
        vo.setOpenDay(fundInfo.getOpenDay());
        vo.setProductName(fundInfo.getFundShortName());
        vo.setDesc(fundInfo.getDescInfo());

        /*// 产品编码
        Map<String, List<IndexesProfile>> indexIdMap = getIndexIdMap(ls);
        getProductCode(indexIdMap, vo, fundInfo);*/

        //查询净值 最新净值 累计净值取分红不投
        {
            PageMethod.startPage(1, 1);
            FundNavExample navExample = new FundNavExample();
            navExample.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
            navExample.setOrderByClause("price_date desc");
            List<FundNav> fundNavList = fundNavMapper.selectByExample(navExample);
            if (CollectionUtils.isNotEmpty(fundNavList)) {
                vo.setNet(Net.builder().netDate(fundNavList.get(0).getPriceDate().getTime())
                        .netValue(BigDecimalUtil.toString(fundNavList.get(0).getNav(), 4))
                        .cumulativeNav(BigDecimalUtil.toString(fundNavList.get(0).getCumulativeNavWithdrawal(), 4))
                        .build());
            }
        }

        // 起投金额
        Map<String, FundRateMapping> rateMappingMap = getStringFundRateMappingMap(Lists.newArrayList(id));
        getMinInvestmentShare(rateMappingMap, fundInfo.getId(), vo);

        FundCountExample example1 = new FundCountExample();
        example1.createCriteria().andFundIdEqualTo(id);
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example1);
        if (CollectionUtils.isNotEmpty(fundCounts)) {
            FundCount fundCount = fundCounts.get(0);
            vo.setRet1m(multiply100(fundCount.getRet1m()));
            vo.setRet1y(multiply100(fundCount.getRet1y()));
            vo.setCumulativeProfit(BigDecimalUtil.toString(fundCount.getCumulativeNav()));
            vo.setRetYtd(multiply100(fundCount.getRetYtd()));
            vo.setRetIncep(multiply100(fundCount.getRetIncep()));
        }

        // 基金公司信息
        if (StringUtils.isNotBlank(fundInfo.getTrustId())) {
            CompanyInfo trust = this.companyInfoMapper.selectByPrimaryKey(fundInfo.getTrustId());
            if (trust != null) {
                vo.setTrustId(trust.getId());
                vo.setTrustName(trust.getCompanyName());
            }
        }

        vo.setIsAppoint(fundInfo.getType());

        vo.setRiskLevel(fundInfo.getRiskLevel());

        vo.setInceptionDate(fundInfo.getInceptionDate() != null ? fundInfo.getInceptionDate().getTime() : null);

        List<FundManagerVO> fundManagerVOList = this.getFundManager(id);
        if (CollectionUtils.isNotEmpty(fundManagerVOList)) {
            FundManagerVO fundManagerVO = fundManagerVOList.get(0);
            vo.setManagerId(fundManagerVO.getManagerId());
            vo.setManagerName(fundManagerVO.getManagerName());
        }

        //子策略名称
        if (vo.getSubstrategy() != null) {
            CommonResp<List<SysConstantResp>> listCommonResp = this.fatools.queryLabels(SysConstEnums.TAMPSUBSTRATEGY.getConstantGroup());
            if (listCommonResp.isSuccess()) {
                listCommonResp.getAttributes().stream()
                        .filter(item -> StringUtils.equals(String.valueOf(vo.getSubstrategy()), item.getConstantCode()))
                        .findFirst()
                        .ifPresent(sysConstantResp -> vo.setStrategyName(sysConstantResp.getConstantName()));
            }
        }

        vo.setProductType(Constant.ONE_NUM);

        if (fundInfo.getFundType() != null) {
            vo.setFundTypeName(ProductEnums.PrivateFundTypeEnum.getByFundType(fundInfo.getFundType()));
        }

        return vo;
    }

    @Override
    public List<ProductInfoVO> getProductList(List<String> ids) {

        List<ProductInfoVO> vos = new ArrayList<>();

        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(ids).andDeleteTagEqualTo(ZERO_NUM);

        List<FundInfo> infos = this.fundInfoMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(infos)) {
            return new ArrayList<>(0);
        }

        FundCountExample example1 = new FundCountExample();
        example1.createCriteria().andFundIdIn(ids);

        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example1);

        Map<String, FundCount> fundCountMap = fundCounts.stream().collect(Collectors.toMap(FundCount::getFundId, s -> s, (oldValue, newValue) -> newValue));

        infos.forEach(info -> {
            ProductInfoVO vo = new ProductInfoVO();
            if (fundCountMap.containsKey(info.getId())) {
                FundCount fundCount = fundCountMap.get(info.getId());
                vo.setRet1m(multiply100(fundCount.getRet1m()));
                vo.setRetIncep(multiply100(fundCount.getRetIncep()));
            }
            BeanUtils.copyProperties(info, vo);
            vo.setOpenDay(info.getOpenDay());
            vo.setProductName(info.getFundShortName());
            vo.setDesc(info.getDescInfo());
            vo.setFundId(info.getId());
            // 最新净值
            vo.setNet(getNewNet(info.getId()));
            vo.setStrategy(info.getStrategy() + "");
            vo.setSubstrategy(info.getSubstrategy() + "");
            vo.setProductType(Constant.ONE_NUM);
            vos.add(vo);
        });

        return vos;
    }

    @Override
    public Page<ProductInfoVO> getNotTanpuProduct(ProductInfoReq req) {

        com.github.pagehelper.Page<ProductInfoVO> page = PageMethod.startPage(req.getPage().getPageNumber(),
                req.getPage().getPageSize());

        FundCountExample example = filterFundCountExample(req);

        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);

        List<ProductInfoVO> vos = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(fundCounts)) {
            FundInfoExample example1 = new FundInfoExample();
            example1.createCriteria()
                    .andIdIn(fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList()))
                    .andDeleteTagEqualTo(ZERO_NUM);
            Map<String, FundInfo> map = this.fundInfoMapper.selectByExample(example1).stream()
                    .collect(Collectors.toMap(FundInfo::getId, f -> f));

            vos = fundCounts.stream().map(f -> getNotTanpuProductInfoVO(map, f)).collect(Collectors.toList());
        }

        return new Page<>(req.getPage(), page.getTotal(), vos);
    }

    private FundCountExample filterFundCountExample(ProductInfoReq req) {
        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria()
//                .andStatusEqualTo(ONE_NUM)
//                .andDataSourcesEqualTo(TWO_NUM)
                .andDeleteTagEqualTo(ZERO_NUM);

        if (!CollectionUtils.isEmpty(req.getList())) {
            criteria.andSubstrategyIn(req.getList().stream().map(Integer::parseInt).collect(Collectors.toList()));
        }

        if (StringUtils.isNotEmpty(req.getNetSort())) {
            if (ONE.equals(req.getNetSort())) {
                example.setOrderByClause("cumulative_nav desc");
            } else {
                example.setOrderByClause("cumulative_nav");
            }
        } else if (StringUtils.isNotEmpty(req.getNear1YearSort())) {
            if (ONE.equals(req.getNear1YearSort())) {
                example.setOrderByClause("ret_1y desc");
            } else {
                example.setOrderByClause("ret_1y");
            }
        } else {
            example.setOrderByClause("cumulative_nav desc");
        }
        return example;
    }

    @Override
    public FundCompanyVO getFundCompany(String id) {
        CompanyInfo companyInfo = this.companyInfoMapper.selectByPrimaryKey(id);
        if (companyInfo == null) {
            return new FundCompanyVO();
        }

        /*CompanyTnaPersonnelExample example = new CompanyTnaPersonnelExample();
        example.createCriteria().andCompanyIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<CompanyTnaPersonnel> companyTnaPersonnels = this.companyTnaPersonnelMapper.selectByExample(example);*/

        FundCompanyVO vo = new FundCompanyVO();
        vo.setCompanyName(companyInfo.getCompanyName());
        vo.setCompanyId(companyInfo.getId());
        vo.setCompanyProfile(companyInfo.getCompanyProfile());
        vo.setImg(filePrefix + companyInfo.getLogo());

        /*if (CollectionUtils.isNotEmpty(companyTnaPersonnels)) {
            CompanyTnaPersonnel companyTnaPersonnel = companyTnaPersonnels.get(0);
            vo.setTotalMoney(companyTnaPersonnel.getTotalAssetSize() == null ? null
                    : companyTnaPersonnel.getTotalAssetSize().toString());
            vo.setTotalFund(companyTnaPersonnel.getFundNum());
        }*/

        // 其他基金信息
        otherFundInfo(vo, id);

        return vo;
    }

    private FundCountExample filterFundCountExample(ProductListReq req) {
        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria()
//                .andStatusEqualTo(ONE_NUM)
//                .andDataSourcesEqualTo(TWO_NUM)
                .andDeleteTagEqualTo(ZERO_NUM);

        if (!CollectionUtils.isEmpty(req.getFundIds())) {
            criteria.andFundIdIn(req.getFundIds());
        }

        if (!CollectionUtils.isEmpty(req.getList())) {
            criteria.andSubstrategyIn(req.getList().stream().map(Integer::parseInt).collect(Collectors.toList()));
        }

        if (StringUtils.isNotEmpty(req.getNetSort())) {
            if (ONE.equals(req.getNetSort())) {
                example.setOrderByClause("cumulative_nav desc");
            } else {
                example.setOrderByClause("cumulative_nav");
            }
        } else if (StringUtils.isNotEmpty(req.getNear1YearSort())) {
            if (ONE.equals(req.getNear1YearSort())) {
                example.setOrderByClause("ret_1y desc");
            } else {
                example.setOrderByClause("ret_1y");
            }
        } else {
            example.setOrderByClause("cumulative_nav desc");
        }
        return example;
    }

    private ProductInfoVO getNotTanpuProductInfoVO(Map<String, FundInfo> map, FundCount f) {
        ProductInfoVO vo = new ProductInfoVO();
        BeanUtils.copyProperties(f, vo);
        vo.setFundId(f.getFundId());
        vo.setNet(new Net(f.getPriceDate().getTime(), BigDecimalUtil.toString(f.getNetNav(), 4), BigDecimalUtil.toString(f.getCumulativeNav(), 4)));
        vo.setProductName(map.get(f.getFundId()).getFundShortName());
        vo.setRet1y(multiply100(f.getRet1y()));

        return vo;
    }

    @Override
    public UnitNetVO getUnitNet(String id, String ratioId, String type) {

        UnitNetVO vo = new UnitNetVO();

        Map<String, Date> timeMap = new HashMap<>();

        filterUnitNetParam(id, ratioId, type, vo, timeMap, null, null);

        FundNavExample example = new FundNavExample();
        FundNavExample.Criteria criteria = example.createCriteria();
        if (timeMap.get("end") != null) {
            criteria.andPriceDateBetween(timeMap.get("end"), timeMap.get("start"));
        }
        criteria.andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("price_date");

        criteria.andFundIdEqualTo(id);
        List<FundNav> navs = this.fundNavMapper.selectByExample(example);

        criteria.andFundIdEqualTo(ratioId);
        List<FundNav> ratioNavs = this.fundNavMapper.selectByExample(example);

        vo.setNets(getNet(navs));

        vo.setRatioNets(getNet(ratioNavs));

        FundInfo fundInfo = this.fundInfoMapper.selectByPrimaryKey(id);
        if (fundInfo != null) {
            vo.setFoundTime(DateUtil.format(fundInfo.getInceptionDate(), DatePattern.NORM_DATE_PATTERN));
        }

        return vo;
    }

    private void filterUnitNetParam(String id, String ratioId, String type, UnitNetVO vo,
                                    Map<String, Date> timeMap, Long beginTime, Long endTime) {
        String today = DateUtil.today();

        Map<String, FundCount> map = getUnitNetRetParam(id, ratioId, vo);

        vo.setTime(DateUtil.format(map.get(id).getEndDate(), DatePattern.NORM_DATE_PATTERN));

        timeMap.put("start", DateUtil.parse(today));
        if (FilterTypeEnum.NEAR_1_MONTH.getCode().equals(type)) {
            timeMap.put("end", DateUtil.offsetMonth(timeMap.get("start"), -1));
            vo.setNet(multiply100(map.get(id).getRet1m()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRet1mBm1()));
            }
        } else if (FilterTypeEnum.NEAR_3_MONTH.getCode().equals(type)) {

            timeMap.put("end", DateUtil.offsetMonth(timeMap.get("start"), -3));
            vo.setNet(multiply100(map.get(id).getRet3m()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRet3mBm1()));
            }
        } else if (FilterTypeEnum.NEAR_6_MONTH.getCode().equals(type)) {

            timeMap.put("end", DateUtil.offsetMonth(timeMap.get("start"), -6));
            vo.setNet(multiply100(map.get(id).getRet6m()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRet6mBm1()));
            }
        } else if (FilterTypeEnum.NEAR_1_YEAR.getCode().equals(type)) {

            timeMap.put("end", DateUtil.offsetMonth(timeMap.get("start"), -12));

            vo.setNet(multiply100(map.get(id).getRet1y()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRet1yBm1()));
            }
        } else if (SINCE_ESTABLISHED.getCode().equals(type)) {
            FundInfo fundInfo = this.fundInfoMapper.selectByPrimaryKey(id);
            timeMap.put("end", fundInfo.getInceptionDate());

            vo.setNet(multiply100(map.get(id).getRetIncep()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRetIncepBm1()));
            }
        } else if (FilterTypeEnum.CUSTOMR.getCode().equals(type)) {
            timeMap.put("start", DateUtil.beginOfDay(DateUtil.date(beginTime)));

            timeMap.put("end", DateUtil.endOfDay(DateUtil.date(endTime)));

            vo.setNet(multiply100(map.get(id).getRet1y()));
            if (map.get(ratioId) != null) {
                vo.setRatioProductNet(multiply100(map.get(id).getRet1yBm1()));
            }
        }
    }

    @Override
    public UnitNetVO getProfitTrend(String id, String ratioId, String type, Long beginTime, Long endTime) {
        UnitNetVO vo = new UnitNetVO();

        Map<String, Date> timeMap = new HashMap<>();

        filterUnitNetParam(id, ratioId, type, vo, timeMap, beginTime, endTime);

        // 成立时间
        FundInfo info = this.fundInfoMapper.selectByPrimaryKey(id);
        vo.setFoundTime(DateUtil.format(info.getInceptionDate(), DatePattern.NORM_DATE_PATTERN));

        FundPerformanceExample example = new FundPerformanceExample();
        FundPerformanceExample.Criteria criteria = example.createCriteria();
        if (timeMap.get("end") != null) {
            criteria.andEndDateBetween(timeMap.get("end"), timeMap.get("start"));
        }
        example.setOrderByClause("end_date");

        criteria.andFundIdIn(Lists.newArrayList(id, ratioId));
        List<FundPerformance> performances = this.fundPerformanceMapper.selectByExample(example);

        if (CollectionUtils.isEmpty(performances)) {
            return vo;
        }
        Map<String, List<FundPerformance>> listMap = performances.stream().collect(Collectors.groupingBy(FundPerformance::getFundId));

        vo.setNets(getProfitNets(listMap.get(id), type, ONE_NUM));
        vo.setRatioNets(getProfitNets(listMap.get(ratioId), type, TWO_NUM));

        return vo;
    }

    @Override
    public Page<NetVO> getNetList(NetReq req) {

        FundCountExample example = getFundCountExample(req);

        com.github.pagehelper.Page<NetVO> page = PageMethod.startPage(req.getPage().getPageNumber(),
                req.getPage().getPageSize());
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);
        List<String> fundIds = fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList());

        if (CollectionUtils.isEmpty(fundIds)) {
            return new Page<>(req.getPage(), page.getTotal(), new ArrayList<>(0));
        }
        FundInfoExample example1 = new FundInfoExample();
        example1.createCriteria().andIdIn(fundIds).andDeleteTagEqualTo(ZERO_NUM);
        Map<String, FundInfo> infoMap = this.fundInfoMapper.selectByExample(example1).stream()
                .collect(Collectors.toMap(FundInfo::getId, f -> f));

        return new Page<>(req.getPage(), page.getTotal(),
                fundCounts.stream().map(f -> getNetVO(infoMap, f)).collect(Collectors.toList()));
    }

    @Override
    public List<BonusRatioVO> getBonusRatio(String id) {
        FundDistributionExample example = new FundDistributionExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("distribute_date desc");
        List<FundDistribution> list = this.fundDistributionMapper.selectByExample(example);
        return list.stream().filter(
                l -> l.getDistributeDate() != null && (l.getDistributeType() == 1 || l.getDistributeType() == 2))
                .map(l -> {
                    BonusRatioVO vo = new BonusRatioVO();
                    vo.setType(l.getDistributeType());
                    vo.setTime(l.getDistributeDate().getTime());
                    vo.setDistribution(BigDecimalUtil.toString(l.getDistribution(), 2));
                    return vo;
                }).collect(Collectors.toList());
    }

    private NetVO getNetVO(Map<String, FundInfo> infoMap, FundCount f) {
        NetVO netVO = new NetVO();
        netVO.setProductId(f.getFundId());
        netVO.setNet(new Net(f.getPriceDate().getTime(), BigDecimalUtil.toString(f.getCumulativeNav(), 4), BigDecimalUtil.toString(f.getCumulativeNav(), 4)));

        FundInfo fundInfo = infoMap.get(f.getFundId());
        netVO.setProductName(fundInfo.getFundShortName());
        netVO.setRatioId(ProductEnums.IndexEnum.getValue(fundInfo.getPrimaryBenchmarkId()));

        netVO.setAboutYearProfit(multiply100(f.getRet6m()));
        netVO.setWeekProfit(multiply100(f.getRet1w()));
        netVO.setNear1MonthProfit(multiply100(f.getRet1m()));
        netVO.setNear1YearProfit(multiply100(f.getRet1y()));

        netVO.setNear1YearVolatility(multiply100(f.getStddev1y()));
        netVO.setNear1YearRetreat(multiply100(f.getMaxdrawdown1y()));

        netVO.setNear1YearSharp(BigDecimalUtil.toString(f.getSharperatio1y()));

        return netVO;
    }

    private FundCountExample getFundCountExample(NetReq req) {
        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria();
//                .andStatusEqualTo(ONE_NUM).andDataSourcesEqualTo(ONE_NUM)
//                .andDeleteTagEqualTo(ZERO_NUM)
//                .andSubstrategyNotEqualTo(ProductEnums.FundSubStrategyEnum.COMBINATION.subStrategy);

        if (CollectionUtils.isNotEmpty(req.getList())) {
            criteria.andSubstrategyIn(req.getList().stream().map(Integer::parseInt).collect(Collectors.toList()));
        }

        if (StringUtils.isNotEmpty(req.getProfitSort())) {
            if (ONE.equals(req.getProfitSort())) {
                example.setOrderByClause("ret_1m desc");
            } else {
                example.setOrderByClause("ret_1m");
            }
        }

        if (StringUtils.isNotEmpty(req.getNetSort())) {
            if (ONE.equals(req.getNetSort())) {
                example.setOrderByClause("cumulative_nav desc");
            } else {
                example.setOrderByClause("cumulative_nav");
            }
        }

        if (StringUtils.isNotEmpty(req.getAboutYearProfitSort())) {
            if (ONE.equals(req.getAboutYearProfitSort())) {
                example.setOrderByClause("ret_6m desc");
            } else {
                example.setOrderByClause("ret_6m");
            }
        }

        if (StringUtils.isNotEmpty(req.getWeekProfitSort())) {
            if (ONE.equals(req.getWeekProfitSort())) {
                example.setOrderByClause("ret_1w desc");
            } else {
                example.setOrderByClause("ret_1w");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1MonthProfitSort())) {
            if (ONE.equals(req.getNear1MonthProfitSort())) {
                example.setOrderByClause("ret_1m desc");
            } else {
                example.setOrderByClause("ret_1m");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1YearProfitSort())) {
            if (ONE.equals(req.getNear1YearProfitSort())) {
                example.setOrderByClause("ret_1y desc");
            } else {
                example.setOrderByClause("ret_1y");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1YearVolatilitySort())) {
            if (ONE.equals(req.getNear1YearVolatilitySort())) {
                example.setOrderByClause("stddev_1y desc");
            } else {
                example.setOrderByClause("stddev_1y");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1YearRetreatSort())) {
            if (ONE.equals(req.getNear1YearRetreatSort())) {
                example.setOrderByClause("maxdrawdown_1y desc");
            } else {
                example.setOrderByClause("maxdrawdown_1y");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1YearSharpSort())) {
            if (ONE.equals(req.getNear1YearSharpSort())) {
                example.setOrderByClause("sharperatio_1y desc");
            } else {
                example.setOrderByClause("sharperatio_1y");
            }
        }
        return example;
    }

    private void getDelegateById(String personalId, Map<String, List<FundManagerMapping>> map,
                                 Map<String, FundInfo> infoMap, Map<String, FundCount> fundCountMap, FundManagerVO vo) {
        if (CollectionUtils.isNotEmpty(map.get(personalId))) {
            List<FundManagerMapping> mappings = map.get(personalId);

            if (CollectionUtils.isEmpty(mappings)) {
                return;
            }

//            List<FundCount> list = mappings.stream()
//                    .filter(mapping -> fundCountMap.containsKey(mapping.getFundId()))
//                    .map(m -> fundCountMap.get(m.getFundId()))
//                    .sorted(Comparator.comparing(FundCount::getRetIncep).reversed())
//                    .collect(Collectors.toList());

            List<FundCount> list = mappings.stream()
                    .filter(mapping -> fundCountMap.containsKey(mapping.getFundId()))
                    .map(m -> fundCountMap.get(m.getFundId())).collect(Collectors.toList());

            if (CollectionUtils.isNotEmpty(list)) {
                FundCount fundCount = list.get(0);
                vo.setRetIncep(multiply100(fundCount.getRetIncep()));
                vo.setInceptionDate(fundCount.getPriceDate() == null ? null : fundCount.getPriceDate().getTime());
            }

            vo.setFundInfoVOS(list.stream().map(l -> {
                FundInfoVO infoVO = new FundInfoVO();
                String fundId = l.getFundId();
                infoVO.setFundId(fundId);
                if (infoMap.get(fundId) != null) {
                    FundInfo fundInfo = infoMap.get(fundId);
                    infoVO.setFundName(fundInfo.getFundShortName());
                    infoVO.setRatioId(ProductEnums.IndexEnum.getValue(fundInfo.getPrimaryBenchmarkId()));
                }
                infoVO.setRetIncep(multiply100(l.getRetIncep()));
                infoVO.setYearProfit(multiply100(l.getRetIncepA()));
                return infoVO;
            }).collect(Collectors.toList()));
        }
    }

    /*@Override
    public List<BonusRatioVO> getBonusRatio(String id) {
        FundDistributionExample example = new FundDistributionExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("distribute_date desc");
        List<FundDistribution> list = this.fundDistributionMapper.selectByExample(example);
        return list.stream().filter(
                l -> l.getDistributeDate() != null && (l.getDistributeType() == 1 || l.getDistributeType() == 2))
                .map(l -> {
                    BonusRatioVO vo = new BonusRatioVO();
                    vo.setType(l.getDistributeType());
                    vo.setTime(l.getDistributeDate().getTime());
                    if (l.getDistribution() != null) {
                        vo.setDistribution(l.getDistribution().setScale(2, BigDecimal.ROUND_HALF_UP).toString());
                    }
                    return vo;
                }).collect(Collectors.toList());
    }*/

    @Override
    public List<TrackRecordVO> getTrackRecord(String id) {
        FundCountExample example = new FundCountExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);

        if (CollectionUtils.isEmpty(fundCounts)) {
            return new ArrayList<>();
        }
        FundCount fundCount = fundCounts.get(0);

        return getTrackRecordVOS(fundCount);
    }

    private ArrayList<TrackRecordVO> getTrackRecordVOS(FundCount fundCount) {
        return Lists.newArrayList(
                new TrackRecordVO("近三个月", multiply100(fundCount.getRet3m()), multiply100(fundCount.getRet3mBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近半年", multiply100(fundCount.getRet6m()), multiply100(fundCount.getRet6mBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近一年", multiply100(fundCount.getRet1y()), multiply100(fundCount.getRet1yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近二年", multiply100(fundCount.getRet2y()), multiply100(fundCount.getRet2yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近三年", multiply100(fundCount.getRet3y()), multiply100(fundCount.getRet3yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近四年", multiply100(fundCount.getRet4y()), multiply100(fundCount.getRet4yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近五年", multiply100(fundCount.getRet5y()), multiply100(fundCount.getRet5yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("成立以来", multiply100(fundCount.getRetIncep()), multiply100(fundCount.getRetIncepBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()));
    }

    @Override
    public Page<TrackNetVO> getTrackNetList(String id, Pageable page) {
        //为计算日涨幅多查一条 返回时要去除
        com.github.pagehelper.Page<TrackNetVO> startPage = PageMethod.startPage(page.getPageNumber(),
                page.getPageSize() + 1);
        FundNavExample example = new FundNavExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("price_date desc");
        List<FundNav> fundNavs = this.fundNavMapper.selectByExample(example);

        // 净值相同日期去重
        fundNavs = fundNavs.stream().collect(
                Collectors.collectingAndThen(
                        Collectors.toCollection(() ->
                                new TreeSet<>(Comparator.comparing(FundNav::getPriceDate))), ArrayList::new))
                .stream().sorted(Comparator.comparing(FundNav::getPriceDate).reversed()).collect(Collectors.toList());

        //计算 日涨幅=(今日净值-前一日净值)/前一日净值
        List<TrackNetVO> list = new ArrayList<>();
        for (int i = 0; i < fundNavs.size(); i++) {
            FundNav f = fundNavs.get(i);

            TrackNetVO vo = new TrackNetVO();
            vo.setDailyIncrease("0.00");
            //日涨幅
            {
                if (i < fundNavs.size() - 1) {
                    FundNav pre = fundNavs.get(i + 1);

                    if (pre.getNav().compareTo(BigDecimal.ZERO) == 1) {
                        BigDecimal dailyIncrease = f.getNav().subtract(pre.getNav()).divide(pre.getNav(), 4, BigDecimal.ROUND_HALF_UP);
                        vo.setDailyIncrease(BigDecimalUtil.multiply100(dailyIncrease));
                    }
                }
            }

            vo.setPriceDate(DateUtil.format(f.getPriceDate(), DatePattern.NORM_DATE_PATTERN));
            vo.setNav(BigDecimalUtil.toString(f.getNav(), 4));
            vo.setCumulativeNav(BigDecimalUtil.toString(f.getCumulativeNav(), 4));
            vo.setCumulativeNavWithdrawal(BigDecimalUtil.toString(f.getCumulativeNavWithdrawal(), 4));
            list.add(vo);
        }

        if (list.size() > page.getPageSize()) {
            //移除最后一条
            list.remove(list.size() - 1);
        }
        return new Page<>(page, startPage.getTotal(), list);
    }

    @Override
    public List<RiskRatingVO> getRiskRating(String id) {
        FundCountExample example = new FundCountExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);

        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(fundCounts)) {
            return new ArrayList<>(0);
        }
        FundCount fundCount = fundCounts.get(0);

        return Lists.newArrayList(
                new RiskRatingVO("最近一年", multiply100(fundCount.getMaxdrawdown1y()),
                        multiply100(fundCount.getStddev1y()), BigDecimalUtil.toString(fundCount.getSharperatio1y()), multiply100(fundCount.getRet1y())),
                new RiskRatingVO("最近二年", multiply100(fundCount.getMaxdrawdown2y()),
                        multiply100(fundCount.getStddev2y()), BigDecimalUtil.toString(fundCount.getSharperatio2y()), multiply100(fundCount.getRet2yA())),
                new RiskRatingVO("最近三年", multiply100(fundCount.getMaxdrawdown3y()),
                        multiply100(fundCount.getStddev3y()), BigDecimalUtil.toString(fundCount.getSharperatio3y()), multiply100(fundCount.getRet3yA())),
                new RiskRatingVO("最近四年", multiply100(fundCount.getMaxdrawdown4y()),
                        multiply100(fundCount.getStddev4y()), BigDecimalUtil.toString(fundCount.getSharperatio4y()), multiply100(fundCount.getRet4yA())),
                new RiskRatingVO("最近五年", multiply100(fundCount.getMaxdrawdown5y()),
                        multiply100(fundCount.getStddev5y()), BigDecimalUtil.toString(fundCount.getSharperatio5y()), multiply100(fundCount.getRet5yA())),
                new RiskRatingVO("成立以来", multiply100(fundCount.getMaxdrawdownIncep()),
                        multiply100(fundCount.getStddevIncep()), BigDecimalUtil.toString(fundCount.getSharperatioIncep()), multiply100(fundCount.getRetIncepA())));
    }

    @Override
    public List<FundManagerVO> getFundManager(String id) {
        FundManagerMappingExample example = new FundManagerMappingExample();
        example.createCriteria().andFundIdEqualTo(id).andIsvisibleEqualTo(ONE_NUM).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("management_start_date desc,id desc");

        List<String> list = this.fundManagerMappingMapper.selectByExample(example).stream()
                .map(FundManagerMapping::getFundManagerId).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>(0);
        }

        example.clear();
        example.createCriteria().andFundManagerIdIn(list);
        List<FundManagerMapping> managerMappings = this.fundManagerMappingMapper.selectByExample(example);

        List<String> fundIds = managerMappings.stream().map(FundManagerMapping::getFundId).distinct()
                .collect(Collectors.toList());

        FundCountExample example2 = new FundCountExample();
        example2.createCriteria().andFundIdIn(fundIds).andDeleteTagEqualTo(ZERO_NUM);
        example2.setOrderByClause("ret_incep desc");
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example2);
        Map<String, FundCount> fundCountMap = fundCounts.stream()
                .collect(Collectors.toMap(FundCount::getFundId, f -> f));

        Map<String, List<FundManagerMapping>> map = managerMappings.stream()
                .collect(Collectors.groupingBy(FundManagerMapping::getFundManagerId));

        // 获取基金基本信息
        FundInfoExample example3 = new FundInfoExample();
        example3.createCriteria().andIdIn(fundIds).andDeleteTagEqualTo(ZERO_NUM);
        Map<String, FundInfo> infoMap = this.fundInfoMapper.selectByExample(example3).stream()
                .collect(Collectors.toMap(FundInfo::getId, f -> f));

        // 获取详情信息
        PersonnelInfoExample example1 = new PersonnelInfoExample();
        example1.createCriteria().andIdIn(list).andDeleteTagEqualTo(ZERO_NUM);
        List<PersonnelInfo> infos = this.personnelInfoMapper.selectByExampleWithBLOBs(example1);

        return infos.stream().map(i -> {
            FundManagerVO vo = new FundManagerVO();
            vo.setManagerId(i.getId());
            vo.setManagerName(i.getPersonnelName());
            vo.setJobTime(i.getInvestmentExperience());
            vo.setPersonalProfile(i.getProfile());

            List<FundManagerMapping> managerMappingList = map.get(i.getId());
            if (CollectionUtils.isNotEmpty(map.get(i.getId()))) {
                String fundId = managerMappingList.get(0).getFundId();
                vo.setId(fundId);

                if (infoMap != null) {
                    if (infoMap.containsKey(fundId)) {
                        FundInfo info = infoMap.get(fundId);
                        vo.setFundName(info.getFundShortName());
                        vo.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));
                    }

                    FundCount fundCount = fundCountMap.get(fundId);
                    if (fundCount != null) {
                        vo.setRetIncep(multiply100(fundCount.getRetIncep()));
                        vo.setNet(new Net(fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime(),
                                BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4)));
                    }

                    if (infoMap.get(fundId) != null) {
                        FundInfo fundInfo = infoMap.get(fundId);
                        vo.setTime(timeLong(fundInfo.getRegisterDate()));
                        vo.setInceptionDate(timeLong(fundInfo.getInceptionDate()));
                    }
                }
                // 设置代表基金id 其他基金信息
                getDelegateById(i.getId(), map, infoMap, fundCountMap, vo);
            }
            return vo;
        }).collect(Collectors.toList());
    }

    @Override
    public FundRecordVO getFundRecord(String id) {
        FundRecordVO vo = new FundRecordVO();

        FundInfo fundInfo = this.fundInfoMapper.selectByPrimaryKey(id);

        FundRateMappingExample example = new FundRateMappingExample();
        example.createCriteria().andFundIdEqualTo(id);
        List<FundRateMapping> fundRateMappings = this.fundRateMappingMapper.selectByExample(example);

        vo.setRegisterNumber(fundInfo.getRegisterNumber());
        CompanyInfoExample example1 = new CompanyInfoExample();
        example1.createCriteria()
                .andIdIn(Lists.newArrayList(fundInfo.getTrustId(), fundInfo.getCustodianId(), fundInfo.getAdvisorId(),
                        fundInfo.getCustodianId(), fundInfo.getLiquidationAgencyId(), fundInfo.getBrokerId(),
                        fundInfo.getBrokerFutureId()))
                .andDeleteTagEqualTo(ZERO_NUM);

        Map<String, CompanyInfo> map = this.companyInfoMapper.selectByExample(example1).stream()
                .collect(Collectors.toMap(CompanyInfo::getId, c -> c));
        vo.setProductName(fundInfo.getFundShortName());
        vo.setId(id);
        vo.setFoundDate(timeLong(fundInfo.getInceptionDate()));
        // TODO 暂时没有 vo.setRiskLevel(fundInfo.);
        if (CollectionUtils.isNotEmpty(fundRateMappings)) {
            vo.setInitialSize(BigDecimalUtil.toString(fundRateMappings.get(0).getInitialSize()));
            vo.setIssue(BigDecimalUtil.toString(fundRateMappings.get(0).getInitialSize()));
        }

        if (map.get(fundInfo.getTrustId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getTrustId());
            vo.setCompanyId(companyInfo.getId());
            vo.setCompanyName(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getCustodianId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getCustodianId());
            vo.setFundCustodianId(companyInfo.getId());
            vo.setFundCustodian(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getAdvisorId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getAdvisorId());
            vo.setAdvisorId(companyInfo.getId());
            vo.setAdvisorName(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getCustodianId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getCustodianId());
            vo.setFundCustodianId(companyInfo.getId());
            vo.setFundCustodian(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getLiquidationAgencyId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getLiquidationAgencyId());
            vo.setLiquidationAgencyId(companyInfo.getId());
            vo.setLiquidationAgencyName(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getBrokerId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getBrokerId());
            vo.setBrokerId(companyInfo.getId());
            vo.setBrokerName(companyInfo.getCompanyName());
        }

        if (map.get(fundInfo.getBrokerFutureId()) != null) {
            CompanyInfo companyInfo = map.get(fundInfo.getBrokerFutureId());
            vo.setBrokerFutureId(companyInfo.getId());
            vo.setBrokerFutureName(companyInfo.getCompanyName());
        }

        FundRateMapping fundRateMapping = this.fundRateMappingMapper.selectByPrimaryKey(id);
        if (fundRateMapping != null) {
            vo.setMinInvestmentShare(BigDecimalUtil.toString(fundRateMapping.getMinInvestmentShare()));
            vo.setSubsequentInvestmentShare(BigDecimalUtil.toString(fundRateMapping.getSubsequentInvestmentShare()));
        }

        // 基金管理人
        {
            List<FundManagerVO> fundManager = this.getFundManager(id);
            if (CollectionUtils.isNotEmpty(fundManager)) {
                FundManagerVO fundManagerVO = fundManager.get(0);
                vo.setFundManagerId(fundManagerVO.getManagerId());
                vo.setFundManagerName(fundManagerVO.getManagerName());
            }
        }

        // 运行状态
        /*{
            FundStatus fundStatus = this.fundStatusMapper.selectByPrimaryKey(id);
            if (fundStatus != null) {
                vo.setFundStatus(fundStatus.getFundStatus());
                vo.setFundStatusValue(FundStatusEnum.getName(fundStatus.getFundStatus()));
            }
        }*/

        // 产品类型
        {
            vo.setFundType(fundInfo.getFundType());
            vo.setFundTypeValue(ProductEnums.FundSubStrategyEnum.getBySubStrategy(fundInfo.getFundType()));
        }

        return vo;
    }

    @Override
    public List<ProductInfoVO> getProductInfolist(ProductListReq req) {

        // 策略集合
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(reqFilterResult(req));

//        List<String> list = fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList());

        List<ProductInfoVO> vos = new ArrayList<>();

        if (CollectionUtils.isNotEmpty(req.getFundIds())) {
            vos = getProductInfoVOS(fundCounts, req.getFundIds());
        }

        return vos;
    }

    @Override
    public List<ProductInfoVO> getNotTanpuProductInfolist(ProductListReq req) {
        FundCountExample example = filterFundCountExample(req);

        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);

        List<ProductInfoVO> vos = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(fundCounts)) {
            FundInfoExample example1 = new FundInfoExample();
            example1.createCriteria()
                    .andIdIn(fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList()))
                    .andDeleteTagEqualTo(ZERO_NUM);
            Map<String, FundInfo> map = this.fundInfoMapper.selectByExample(example1).stream()
                    .collect(Collectors.toMap(FundInfo::getId, f -> f));

            vos = fundCounts.stream().map(f -> getNotTanpuProductInfoVO(map, f)).collect(Collectors.toList());
        }

        return vos;
    }

    private void otherFundInfo(FundCompanyVO vo, String id) {

        FundInfoExample example1 = new FundInfoExample();
        example1.createCriteria().andTrustIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<FundInfo> fundInfos = this.fundInfoMapper.selectByExample(example1);
        List<String> list = fundInfos.stream().map(FundInfo::getId).collect(Collectors.toList());
        Map<String, FundInfo> mapFundInfo = fundInfos.stream().collect(Collectors.toMap(FundInfo::getId, f -> f));

        if (CollectionUtils.isNotEmpty(list)) {
            FundCountExample example2 = new FundCountExample();
            example2.createCriteria().andFundIdIn(list).andDeleteTagEqualTo(ZERO_NUM);
            example2.setOrderByClause("ret_incep desc,ret_incep_a desc");
            List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example2);

            if (CollectionUtils.isNotEmpty(fundCounts)) {
                fundCounts.sort(Comparator.comparing(FundCount::getRetIncep, (s1, s2) -> {
                    if (s1 == null) {
                        s1 = BigDecimal.ZERO;
                    }
                    if (s2 == null) {
                        s2 = BigDecimal.ZERO;
                    }
                    return s1.compareTo(s2);
                }).reversed());
                FundCount fundCount = fundCounts.get(0);

                FundInfo info = mapFundInfo.get(fundCount.getFundId());
                vo.setTime(info.getRegisterDate() == null ? null : info.getRegisterDate().getTime());
                vo.setFundId(fundCount.getFundId());
                vo.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));
                vo.setInceptionDate(info.getInceptionDate() == null ? null : info.getInceptionDate().getTime());
                vo.setFundName(info.getFundShortName());
                vo.setNet(new Net(fundCount.getEndDate().getTime(), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4), BigDecimalUtil.toString(fundCount.getCumulativeNav(), 4)));

                String retIncep = multiply100(fundCount.getRetIncep());
                vo.setRetIncep(retIncep == null ? "0" : retIncep);
                vo.setFundInfoVOS(fundCounts.stream().map(f -> {
                    FundInfoVO vo1 = new FundInfoVO();
                    String retIncepstr = multiply100(f.getRetIncep());
                    vo1.setRetIncep(retIncepstr == null ? "0" : retIncepstr);
                    String yearProfit = multiply100(f.getRetIncepA());
                    vo1.setYearProfit(yearProfit == null ? "0" : yearProfit);
                    vo1.setFundName(mapFundInfo.get(f.getFundId()).getFundShortName());
                    vo1.setFundId(f.getFundId());
                    vo1.setRatioId(ProductEnums.IndexEnum.getValue(info.getPrimaryBenchmarkId()));
                    return vo1;
                }).collect(Collectors.toList()));
            }
        }
    }

    private List<Net> getProfitNets(List<FundPerformance> performances, String type, Integer ratioType) {
        if (CollectionUtils.isNotEmpty(performances)) {
            if (!(SINCE_ESTABLISHED.getCode().equals(type) && ratioType.equals(ONE_NUM))) {
                FundPerformance fundPerformance = new FundPerformance();
                BeanUtils.copyProperties(performances.get(0), fundPerformance);
                performances = performances.stream().peek(p -> {
                    p.setRet1m(subtract(p.getRet1m(), fundPerformance.getRet1m()));
                    p.setRet3m(subtract(p.getRet3m(), fundPerformance.getRet3m()));
                    p.setRet6m(subtract(p.getRet6m(), fundPerformance.getRet6m()));
                    p.setRet1y(subtract(p.getRet1y(), fundPerformance.getRet1y()));
                    p.setRetIncep(subtract(p.getRetIncep(), fundPerformance.getRetIncep()));
                }).collect(Collectors.toList());
            }
        }
        if (CollectionUtils.isEmpty(performances)) {
            return new ArrayList<>(0);
        }
        return performances.stream().filter(p -> p.getEndDate() != null).map(p -> {
            Net net = new Net();
            net.setNetDate(p.getEndDate().getTime());
            if (FilterTypeEnum.NEAR_1_MONTH.getCode().equals(type)) {
                net.setNetValue(multiply100(p.getRet1m()));
            } else if (FilterTypeEnum.NEAR_3_MONTH.getCode().equals(type)) {
                net.setNetValue(multiply100(p.getRet3m()));
            } else if (FilterTypeEnum.NEAR_6_MONTH.getCode().equals(type)) {
                net.setNetValue(multiply100(p.getRet6m()));
            } else if (FilterTypeEnum.NEAR_1_YEAR.getCode().equals(type)) {
                net.setNetValue(multiply100(p.getRet1y()));
            } else {
                net.setNetValue(multiply100(p.getRetIncep()));
            }
            return net;
        }).collect(Collectors.toList());
    }

    private Map<String, FundCount> getUnitNetRetParam(String id, String ratioId, UnitNetVO vo) {
        FundCountExample example = new FundCountExample();
        example.createCriteria().andFundIdIn(Lists.newArrayList(id, ratioId)).andDeleteTagEqualTo(ZERO_NUM);

        Map<String, FundCount> map = this.fundCountMapper.selectByExample(example).stream()
                .collect(Collectors.toMap(FundCount::getFundId, f -> f));

        FundInfo info = this.fundInfoMapper.selectByPrimaryKey(ratioId);

        if (info != null) {
            vo.setRatioProductName(info.getFundShortName());
        }
        return map;
    }

    private List<Net> getNet(List<FundNav> navs) {
        return navs.stream().filter(r -> r.getPriceDate() != null).map(r -> {
            Net net = new Net();
            net.setNetDate(r.getPriceDate().getTime());
            net.setNetValue(BigDecimalUtil.toString(r.getNav(), 4));
            return net;
        }).collect(Collectors.toList());
    }

    private List<ProductInfoVO> getProductInfoVOS(List<FundCount> fundCounts, List<String> list) {
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(list).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("create_time desc");

        List<FundInfo> fundInfos = this.fundInfoMapper.selectByExample(example);
//        Map<String, FundInfo> fundInfoMap = fundInfos.stream().collect(Collectors.toMap(FundInfo::getId, f -> f));
        Map<String, FundCount> fundCountMap = fundCounts.stream().collect(Collectors.toMap(FundCount::getFundId, f -> f));

        // 指数indexId
        /*List<String> ls = fundInfos.stream().filter(f -> StringUtils.isNotEmpty(f.getPrimaryBenchmarkId()))
                .map(FundInfo::getPrimaryBenchmarkId).collect(Collectors.toList());

        Map<String, List<IndexesProfile>> indexIdMap = getIndexIdMap(ls);*/

        // 起投金额
        Map<String, FundRateMapping> rateMappingMap = getStringFundRateMappingMap(list);

        // 查询研报信息是否有研报
//        Map<String, Object> fundInfoReport = getFundReport(Lists.newArrayList(fundInfoMap.keySet()));

        //查询净值
        Map<String, FundNav> fundNavMap;
        {
            List<FundNav> navList = fundInfoCustomMapper.getFundInfoNewNet(list);
            if (CollectionUtils.isNotEmpty(navList)) {
                fundNavMap = navList.stream().collect(Collectors.toMap(FundNav::getFundId, c -> c));
            } else {
                fundNavMap = null;
            }
        }

        return fundInfos.stream().map(p -> getProductInfoVO(fundCountMap, rateMappingMap, p, fundNavMap))
                .collect(Collectors.toList());
    }

//    private Map<String, Object> getFundReport(List<String> list) {
//        String ids = list.stream().collect(Collectors.joining("','", "'", "'"));
//        return fundInfoCustomMapper.getFundInfoReport(ids);
//    }

    private ProductInfoVO getProductInfoVO(Map<String, FundCount> fundCountMap, Map<String, FundRateMapping> rateMappingMap,
                                           FundInfo fundInfo,
                                           Map<String, FundNav> fundNavMap) {
        ProductInfoVO vo = new ProductInfoVO();
        if (fundCountMap.containsKey(fundInfo.getId())) {
            FundCount fundCount = fundCountMap.get(fundInfo.getId());

            BeanUtils.copyProperties(fundCount, vo);
            vo.setRet1m(multiply100(fundCount.getRet1m()));
            vo.setRet1y(multiply100(fundCount.getRet1y()));
            vo.setRetIncep(multiply100(fundCount.getRetIncep()));
            vo.setCumulativeProfit(BigDecimalUtil.toString(fundCount.getCumulativeNav()));
        }
        vo.setProductType(1);
        String fundId = fundInfo.getId();
        vo.setFundId(fundId);

        // 指数id
        vo.setRatioId(ProductEnums.IndexEnum.getValue(fundInfo.getPrimaryBenchmarkId()));
        vo.setProductName(fundInfo.getFundShortName());
        // getProductCode(indexIdMap, vo, fundInfo);
        vo.setOpenDay(fundInfo.getOpenDay());
        vo.setDesc(fundInfo.getDescInfo());

        // 风险等级
        vo.setRiskLevel(fundInfo.getRiskLevel());

        vo.setIsReport(0);
        vo.setIsAppoint(fundInfo.getType());

        // 最新净值
        if (fundNavMap != null && fundNavMap.containsKey(fundId)) {
            FundNav fundNav = fundNavMap.get(fundId);
            vo.setNet(new Net(fundNav.getPriceDate().getTime(),
                    BigDecimalUtil.toString(fundNav.getNav(), 4),
                    BigDecimalUtil.toString(fundNav.getCumulativeNavWithdrawal(), 4)));
        }

        getMinInvestmentShare(rateMappingMap, fundInfo.getId(), vo);

        return vo;
    }

    private void getMinInvestmentShare(Map<String, FundRateMapping> rateMappingMap, String fundId, ProductInfoVO vo) {
        FundRateMapping fundRateMapping = rateMappingMap.get(fundId);
        if (fundRateMapping != null) {
            vo.setInitialSize(BigDecimalUtil.toString(fundRateMapping.getInitialSize()));
            vo.setMoney(BigDecimalUtil.toStringNotDecimal(fundRateMapping.getMinInvestmentShare()));
            vo.setMinMoney(BigDecimalUtil.toString(fundRateMapping.getSubsequentInvestmentShare()));
        }
    }

    private Net getNewNet(String fundId) {
        FundNavExample example = new FundNavExample();
        example.createCriteria().andFundIdEqualTo(fundId);
        example.setOrderByClause("price_date desc limit 1");
        List<FundNav> navs = this.fundNavMapper.selectByExample(example);
        Net net = new Net();

        if (CollectionUtils.isNotEmpty(navs)) {
            net.setNetDate(navs.get(0).getPriceDate().getTime());
            net.setNetValue(BigDecimalUtil.toString(navs.get(0).getNav(), 4));
            net.setCumulativeNav(BigDecimalUtil.toString(navs.get(0).getCumulativeNav(), 4));
        }
        return net;
    }

    /*private void getProductCode(Map<String, List<IndexesProfile>> indexIdMap, ProductInfoVO vo, FundInfo fundInfo) {
        if (CollectionUtils.isNotEmpty(indexIdMap.get(fundInfo.getPrimaryBenchmarkId()))) {
            String indexCode = indexIdMap.get(fundInfo.getPrimaryBenchmarkId()).get(0).getIndexCode();
            if (StringUtils.isNotEmpty(indexCode)) {
                vo.setProductCode(indexCode.substring(0, indexCode.indexOf(".")));
            }
        }
    }*/

    private Map<String, FundRateMapping> getStringFundRateMappingMap(List<String> ls) {
        if (CollectionUtils.isEmpty(ls)) {
            return new HashMap<>(0);
        }
        FundRateMappingExample example = new FundRateMappingExample();
        example.createCriteria().andFundIdIn(ls).andDeleteTagEqualTo(ZERO_NUM);
        return this.fundRateMappingMapper.selectByExample(example).stream()
                .collect(Collectors.toMap(FundRateMapping::getFundId, f -> f));
    }

    /*private Map<String, List<IndexesProfile>> getIndexIdMap(List<String> ls) {
        if (CollectionUtils.isEmpty(ls)) {
            return new HashMap<>(0);
        }
        IndexesProfileExample example2 = new IndexesProfileExample();
        example2.createCriteria().andIdIn(ls).andDeleteTagEqualTo(ZERO_NUM);
        return this.indexesProfileMapper.selectByExample(example2).stream()
                .collect(Collectors.groupingBy(IndexesProfile::getIndexCode));
    }*/

    /**
     * 条件筛选数据
     *
     * @param req 入参
     * @return 返回筛选条件
     */
    private FundCountExample reqFilterResult(ProductInfoReq req) {

        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria().andDataSourcesEqualTo(ONE_NUM)
//                .andStatusEqualTo(ONE_NUM)
//                .andSubstrategyNotEqualTo(ProductEnums.FundSubStrategyEnum.COMBINATION.subStrategy)
                .andDeleteTagEqualTo(ZERO_NUM);

        if (CollectionUtils.isNotEmpty(req.getList())) {
            criteria.andSubstrategyIn(req.getList().stream().map(Integer::parseInt).collect(Collectors.toList()));
        }

        if (StringUtils.isNotEmpty(req.getRetIncepSort())) {
            if (ONE.equals(req.getRetIncepSort())) {
                example.setOrderByClause("ret_incep desc");
            } else {
                example.setOrderByClause("ret_incep");
            }
        }

        if (StringUtils.isNotEmpty(req.getNear1YearSort())) {
            if (ONE.equals(req.getNear1YearSort())) {
                example.setOrderByClause("ret_1y desc");
            } else {
                example.setOrderByClause("ret_1y");
            }
        }

        //默认按成立以来收益倒序排序
        if (StringUtils.isEmpty(example.getOrderByClause())) {
            example.setOrderByClause("ret_incep desc");
        }

        return example;
    }

    private FundCountExample reqFilterResult(ProductListReq req) {

        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria().andDataSourcesEqualTo(ONE_NUM)
//                .andStatusEqualTo(ONE_NUM)
//                .andSubstrategyNotEqualTo(ProductEnums.FundSubStrategyEnum.COMBINATION.subStrategy)
                .andDeleteTagEqualTo(ZERO_NUM);

        if (!CollectionUtils.isEmpty(req.getFundIds())) {
            criteria.andFundIdIn(req.getFundIds());
        }

        if (CollectionUtils.isNotEmpty(req.getList())) {
            criteria.andSubstrategyIn(req.getList().stream().map(Integer::parseInt).collect(Collectors.toList()));
        }
        if (StringUtils.isNotEmpty(req.getSort())) {
            if (ONE.equals(req.getSort())) {
                example.setOrderByClause("ret_1m desc");
            } else {
                example.setOrderByClause("ret_1m");
            }
        } else {
            example.setOrderByClause("ret_1m desc");
        }
        return example;
    }

    @Override
    public DynamicRetreatVO getDynamicRetreat(String id) {
        FundRiskStatsExample example = new FundRiskStatsExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("end_date desc");
        DynamicRetreatVO vo = new DynamicRetreatVO();

        List<FundRiskStats> fundRiskStats = this.fundRiskStatsMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(fundRiskStats)) {
            List<Net> nets = new ArrayList<>();
            Optional<FundRiskStats> maxOptional = fundRiskStats.stream()
                    .peek(f ->
                            nets.add(Net.builder()
                                    .netDate(f.getEndDate().getTime())
                                    .netValue(multiply100(f.getMaxdrawdownIncep()))
                                    .build()))
                    .max(Comparator.comparing(FundRiskStats::getMaxdrawdownIncep));
            vo.setNets(nets);
            vo.setHistoryMaxRet(maxOptional
                    .map(riskStats ->
                            Net.builder()
                                    .netValue(multiply100(riskStats.getMaxdrawdownIncep()))
                                    .netDate(riskStats.getEndDate().getTime())
                                    .build())
                    .orElse(null)
            );
            vo.setByTime(nets.get(nets.size() - 1).getNetDate());
        }
        return vo;
    }

    @Override
    public ProductInfoVO getPrivateDetail(String id) {
        IfaImportedFundInfo fundInfo = ifaImportedFundInfoMapper.selectByPrimaryKey(id);

        //查询收益率
        IfaImportedFundCountExample example = new IfaImportedFundCountExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        List<IfaImportedFundCount> fundCountList = ifaImportedFundCountMapper.selectByExample(example);

        ProductInfoVO detailResp = new ProductInfoVO();
        detailResp.setFundId(fundInfo.getId());
        detailResp.setFundName(fundInfo.getFundName());
        detailResp.setProductName(fundInfo.getFundName());
        detailResp.setRegisterNumber(fundInfo.getRegisterNumber());

        if (CollectionUtils.isNotEmpty(fundCountList)) {
            detailResp.setNear1YearProfit(BigDecimalUtil.multiply100(fundCountList.get(0).getRet1y()));
            detailResp.setRet1y(BigDecimalUtil.multiply100(fundCountList.get(0).getRet1y()));
            detailResp.setRetYtd(BigDecimalUtil.multiply100(fundCountList.get(0).getRetYtd()));
        }

        //子策略名称
        if (detailResp.getSubstrategy() != null) {
            CommonResp<List<SysConstantResp>> listCommonResp = this.fatools.queryLabels(SysConstEnums.TAMPSUBSTRATEGY.getConstantGroup());
            if (listCommonResp.isSuccess()) {
                listCommonResp.getAttributes().stream()
                        .filter(item -> StringUtils.equals(String.valueOf(detailResp.getSubstrategy()), item.getConstantCode())).
                        findFirst()
                        .ifPresent(sysConstantResp -> detailResp.setStrategyName(sysConstantResp.getConstantName()));
            }
        }

        //查询基金净值
        {
            PageMethod.startPage(1, 1);
            IfaImportedFundNavExample navExample = new IfaImportedFundNavExample();
            navExample.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
            navExample.setOrderByClause("price_date");
            List<IfaImportedFundNav> navList = ifaImportedFundNavMapper.selectByExample(navExample);
            if (CollectionUtils.isNotEmpty(navList)) {
                detailResp.setNet(Net.builder().netDate(navList.get(0).getPriceDate().getTime())
                        .netValue(BigDecimalUtil.toString(navList.get(0).getCumulativeNav(), 4)).build());
            }
        }

        detailResp.setProductType(Constant.THREE_NUM);

        return detailResp;
    }

    @Override
    public List<TrackRecordVO> getHistoryprofit(String id) {
        IfaImportedFundCountExample example = new IfaImportedFundCountExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        List<IfaImportedFundCount> fundCounts = this.ifaImportedFundCountMapper.selectByExample(example);

        if (CollectionUtils.isEmpty(fundCounts)) {
            return new ArrayList<>();
        }
        IfaImportedFundCount fundCount = fundCounts.get(0);

        return getPrivateFundTrackRecordVOS(fundCount);
    }

    @Override
    public Page<TrackNetVO> getPrivateFundTrackNetList(String id, Pageable page) {
        com.github.pagehelper.Page<TrackNetVO> startPage = PageMethod.startPage(page.getPageNumber(),
                page.getPageSize());
        IfaImportedFundNavExample example = new IfaImportedFundNavExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);
        example.setOrderByClause("price_date desc");
        List<IfaImportedFundNav> fundNavs = this.ifaImportedFundNavMapper.selectByExample(example);

        // 净值相同日期去重
        fundNavs = fundNavs.stream().collect(
                Collectors.collectingAndThen(
                        Collectors.toCollection(() ->
                                new TreeSet<>(Comparator.comparing(IfaImportedFundNav::getPriceDate))), ArrayList::new));

        //计算 日涨幅=(今日净值-前一日净值)/前一日净值
        List<TrackNetVO> list = new ArrayList<>();
        for (int i = 0; i < fundNavs.size(); i++) {
            IfaImportedFundNav f = fundNavs.get(i);

            TrackNetVO vo = new TrackNetVO();
            vo.setDailyIncrease("0.00");
            //日涨幅
            {
                if (i < fundNavs.size() - 1) {
                    IfaImportedFundNav pre = fundNavs.get(i + 1);

                    if (pre.getNav().compareTo(BigDecimal.ZERO) == 1) {
                        BigDecimal dailyIncrease = f.getNav().subtract(pre.getNav()).divide(pre.getNav(), 4, BigDecimal.ROUND_HALF_UP);
                        vo.setDailyIncrease(BigDecimalUtil.multiply100(dailyIncrease));
                    }
                }
            }

            vo.setPriceDate(DateUtil.format(f.getPriceDate(), DatePattern.NORM_DATE_PATTERN));
            vo.setNav(BigDecimalUtil.toString(f.getNav(), 4));
            vo.setCumulativeNav(BigDecimalUtil.toString(f.getCumulativeNav(), 4));
            vo.setCumulativeNavWithdrawal(BigDecimalUtil.toString(f.getCumulativeNav(), 4));
            list.add(vo);
        }

        if (list.size() > page.getPageSize()) {
            //移除最后一条
            list.remove(list.size() - 1);
        }
        return new Page<>(page, startPage.getTotal(), list);
    }

    @Override
    public List<RiskRatingVO> getPrivateFundRiskRating(String id) {
        IfaImportedFundCountExample example = new IfaImportedFundCountExample();
        example.createCriteria().andFundIdEqualTo(id).andDeleteTagEqualTo(ZERO_NUM);

        List<IfaImportedFundCount> fundCounts = this.ifaImportedFundCountMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(fundCounts)) {
            return new ArrayList<>(0);
        }
        IfaImportedFundCount fundCount = fundCounts.get(0);

        return Lists.newArrayList(
                new RiskRatingVO("最近一年", multiply100(fundCount.getMaxdrawdown1y()),
                        multiply100(fundCount.getStddev1y()), multiply100(fundCount.getSharperatio1y()), multiply100(fundCount.getRet1y())),
                new RiskRatingVO("最近二年", multiply100(fundCount.getMaxdrawdown2y()),
                        multiply100(fundCount.getStddev2y()), multiply100(fundCount.getSharperatio2y()), multiply100(fundCount.getRet2yA())),
                new RiskRatingVO("最近三年", multiply100(fundCount.getMaxdrawdown3y()),
                        multiply100(fundCount.getStddev3y()), multiply100(fundCount.getSharperatio3y()), multiply100(fundCount.getRet3yA())),
                new RiskRatingVO("最近四年", multiply100(fundCount.getMaxdrawdown4y()),
                        multiply100(fundCount.getStddev4y()), multiply100(fundCount.getSharperatio4y()), multiply100(fundCount.getRet4yA())),
                new RiskRatingVO("最近五年", multiply100(fundCount.getMaxdrawdown5y()),
                        multiply100(fundCount.getStddev5y()), multiply100(fundCount.getSharperatio5y()), multiply100(fundCount.getRet5yA())),
                new RiskRatingVO("成立以来", multiply100(fundCount.getMaxdrawdownIncep()),
                        multiply100(fundCount.getStddevIncep()), multiply100(fundCount.getSharperatioIncep()), multiply100(fundCount.getRetIncepA())));
    }

    @Override
    public List<ProductInfoVO> getPrivateFundList(List<String> ids) {

        List<IfaImportedFundInfo> fundInfoList;
        {
            IfaImportedFundInfoExample example = new IfaImportedFundInfoExample();
            example.createCriteria().andIdIn(ids).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
            fundInfoList = ifaImportedFundInfoMapper.selectByExample(example);
        }

        Map<String, IfaImportedFundCount> fundCountMap;
        {
            IfaImportedFundCountExample example = new IfaImportedFundCountExample();
            example.createCriteria().andFundIdIn(ids).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
            List<IfaImportedFundCount> countList = ifaImportedFundCountMapper.selectByExample(example);
            if (CollectionUtils.isNotEmpty(countList)) {
                fundCountMap = countList.stream().collect(Collectors.toMap(IfaImportedFundCount::getFundId, c -> c));
            } else {
                fundCountMap = null;
            }
        }

        if (CollectionUtils.isNotEmpty(fundInfoList)) {
            return fundInfoList.stream().map(item -> {
                IfaImportedFundCount importedFundCount = null;
                if (fundCountMap != null && fundCountMap.containsKey(item.getId())) {
                    importedFundCount = fundCountMap.get(item.getId());
                }

                ProductInfoVO p = new ProductInfoVO();
                p.setOpenDay(item.getOpenDay());
                p.setFundId(item.getId());
                p.setSubstrategy(item.getSubstrategy() + "");
                p.setStrategy(item.getStrategy() + "");
                p.setProductName(item.getFundName());
                p.setRet1m(importedFundCount == null ? null : BigDecimalUtil.multiply100(importedFundCount.getRet1m()));
                p.setRetIncep(importedFundCount == null ? null : BigDecimalUtil.multiply100(importedFundCount.getRetIncep()));
                p.setNet(importedFundCount == null ? null : Net.builder().netDate(importedFundCount.getPriceDate().getTime())
                        .netValue(BigDecimalUtil.toString(importedFundCount.getNetNav(), 4))
                        .build());

                p.setProductType(Constant.THREE_NUM);
                return p;
            }).collect(Collectors.toList());
        }

        return new ArrayList<>();
    }

    @Override
    public List<FundInfoSimpleListResp> getSimpleList(List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return Lists.newArrayListWithCapacity(0);
        }
        FundInfoExample example = new FundInfoExample();
        example.createCriteria().andIdIn(idList);
        List<FundInfo> fundInfoList = fundInfoMapper.selectByExample(example);
        return fundInfoList.stream().map(fundInfo -> FundInfoSimpleListResp.builder()
                .id(fundInfo.getId())
                .fundName(fundInfo.getFundName())
                .fundShortName(fundInfo.getFundShortName())
                .build()).collect(Collectors.toList());
    }

    @Override
    public List<FundInfoSimpleListResp> getPrivateSimpleList(List<String> idList) {
        if (CollectionUtils.isEmpty(idList)) {
            return Lists.newArrayListWithCapacity(0);
        }
        IfaImportedFundInfoExample example = new IfaImportedFundInfoExample();
        example.createCriteria().andIdIn(idList);
        List<IfaImportedFundInfo> ifaImportedFundInfoList = ifaImportedFundInfoMapper.selectByExample(example);
        return ifaImportedFundInfoList.stream().map(fundInfo -> FundInfoSimpleListResp.builder()
                .id(fundInfo.getId())
                .fundName(fundInfo.getFundName())
                .fundShortName(fundInfo.getFundName())
                .build())
                .collect(Collectors.toList());
    }

    @Override
    public List<FundSameResp> querySamefund(String fundId) {
        //查询同类基金
        FundInfo fundInfo = fundInfoMapper.selectByPrimaryKey(fundId);

        PageMethod.startPage(1, 10);
        FundCountExample example = new FundCountExample();
        example.createCriteria().andSubstrategyEqualTo(fundInfo.getSubstrategy()).andIdNotEqualTo(fundId)
                .andDataSourcesEqualTo(Constant.ONE_NUM).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        example.setOrderByClause("ret_1y desc");

        List<FundCount> fundCounts = fundCountMapper.selectByExample(example);

        if (CollectionUtils.isNotEmpty(fundCounts)) {
            List<String> fundIdList = fundCounts.stream().map(FundCount::getFundId).collect(Collectors.toList());
            FundInfoExample infoExample = new FundInfoExample();
            infoExample.createCriteria().andIdIn(fundIdList);
            Map<String, String> fundMap = fundInfoMapper.selectByExample(infoExample).stream().collect(Collectors.toMap(FundInfo::getId, FundInfo::getFundShortName));

            List<String> checkFundIdList = feignClientForDiagnose.getFundCollectInfo(fundIdList).getAttributes();

            return fundCounts.stream().map(item -> FundSameResp.builder()
                    .fundId(item.getFundId())
                    .fundName(fundMap.get(item.getFundId()))
                    .ret1y(multiply100(item.getRet1y()))
                    .isCheck(CollectionUtils.isNotEmpty(checkFundIdList) ? (checkFundIdList.contains(item.getFundId()) ? 1 : 0) : 0)
                    .build()).collect(Collectors.toList());
        }

        return new ArrayList<>(0);
    }

    @Override
    public List<FundSameResp> getRemmendPrivateFundList(String fundId) {
        IfaImportedFundInfoExample example = new IfaImportedFundInfoExample();
        example.createCriteria().andIfaIdEqualTo(UserInfoThreadLocalHolder.getserInfo().getId())
                .andIdNotEqualTo(fundId)
                .andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        List<IfaImportedFundInfo> importedFundInfoList = ifaImportedFundInfoMapper.selectByExample(example);

        List<String> fundIdList = importedFundInfoList.stream().map(IfaImportedFundInfo::getId).collect(Collectors.toList());

        IfaImportedFundCountExample countExample = new IfaImportedFundCountExample();
        countExample.createCriteria().andFundIdIn(fundIdList).andDeleteTagEqualTo(BizEnums.DeleteTag.tag_init);
        List<IfaImportedFundCount> importedFundCountList = ifaImportedFundCountMapper.selectByExample(countExample);

        Map<String, IfaImportedFundCount> fundCountMap;
        if (CollectionUtils.isNotEmpty(importedFundCountList)) {
            fundCountMap = importedFundCountList.stream().collect(Collectors.toMap(IfaImportedFundCount::getFundId, c -> c));
        } else {
            fundCountMap = null;
        }

        List<String> checkFundIdList = feignClientForDiagnose.getFundCollectInfo(fundIdList).getAttributes();

        importedFundInfoList.forEach(item -> {
            FundSameResp p = FundSameResp.builder()
                    .fundId(item.getId())
                    .fundName(item.getFundName())
                    .isCheck(CollectionUtils.isNotEmpty(checkFundIdList) ? (checkFundIdList.contains(item.getId()) ? 1 : 0) : 0)
                    .build();

            if (fundCountMap != null && fundCountMap.containsKey(item.getId())) {
                p.setRet1y(BigDecimalUtil.toString(fundCountMap.get(item.getId()).getRet1y(), 2));
            }
        });

        return new ArrayList<>(0);
    }

    @Override
    public List<Type> getSimpleFundList(List<String> list) {
        return fundInfoCustomMapper.getSimpleFundList(list.stream().collect(Collectors.joining("','", "'", "'")));
    }

    @Override
    public CommonResp<List<FundNavMessage>> importNetCheckSave(IfaNetImportReq req) {

        Map<Long, IfaImportedFundNav> ifaNavMap = getIfaNavMap(req.getFundId());

        List<FundNavMessage> resultMessage = new ArrayList<>(req.getFundNavMessages().size() * 2);
        AtomicBoolean errorStatus = new AtomicBoolean(false);


        List<List<Object>> read = req.getFundNavMessages().stream().map(ls -> {
            List<Object> linkedList = new LinkedList<>();
            linkedList.add(ls.getPriceDate() != null ? ls.getPriceDate().getTime() : "");
            linkedList.add(ls.getNav());
            linkedList.add(ls.getCumulativeNav());
            return linkedList;
        }).collect(Collectors.toList());

        return getListCommonResp(ifaNavMap, resultMessage, errorStatus, read);
    }

    @Override
    public Page<FundRankResp> getFundRank(FundRankReq req) {
        FundCountExample example = filterFundCountReq(req);
        Pageable pageable = new Pageable(req.getPageNumber(), req.getPageSize());

        com.github.pagehelper.Page<Object> objects = PageMethod.startPage(pageable);
        List<FundCount> fundCounts = this.fundCountMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(fundCounts)) {
            return new Page<>(pageable, new ArrayList<>(0));
        }

        FundInfoExample example1 = new FundInfoExample();
        List<String> ids = fundCounts
                .stream()
                .map(FundCount::getFundId)
                .collect(Collectors.toList());
        example1.createCriteria().andIdIn(ids);
        Map<String, String> fundNameMap = this.fundInfoMapper.selectByExample(example1)
                .stream()
                .collect(Collectors.toMap(FundInfo::getId, FundInfo::getFundShortName));

        Set<String> fundSets = product.getSimpleFundList(ids)
                .getAttributes()
                .stream()
                .map(Type::getId)
                .collect(Collectors.toSet());

        return new Page<>(pageable, objects.getTotal(), fundCounts
                .stream()
                .map(ls -> {
                    FundRankResp resp = new FundRankResp();
                    String fundId = ls.getFundId();
                    resp.setProductName(Type.builder().id(fundId).value(fundNameMap.get(fundId)).build());

                    boolean productType = true;
                    if (fundSets.contains(fundId)) {
                        productType = false;
                    }
                    resp.setProductType(Type.builder().id(productType ? "1" : "2").value(productType ? "私募基金" : "探普白名单").build());
                    if (ls.getEndDate() != null) {
                        resp.setNet(Net.builder().netValue(BigDecimalUtil.toString(ls.getNetNav(), 4)).netDate(ls.getEndDate().getTime()).build());
                    }

                    if (req.getRank() == null) {
                        resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet1y()));
                    } else {
                        switch (req.getRank()) {
                            case "day":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet1day()));
                                break;
                            case "week":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet1w()));
                                break;
                            case "month":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet1m()));
                                break;
                            case "season":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet3m()));
                                break;
                            case "halfAYear":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet6m()));
                                break;
                            case "year":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet1y()));
                                break;
                            case "twoYear":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet2y()));
                                break;
                            case "threeYear":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet3y()));
                                break;
                            case "fiveYear":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRet5y()));
                                break;
                            case "retIncep":
                                resp.setRangeOfIncrease(BigDecimalUtil.multiply100(ls.getRetIncep()));
                                break;
                        }
                    }
                    return resp;
                }).collect(Collectors.toList()));
    }

    private FundCountExample filterFundCountReq(FundRankReq req) {
        FundCountExample example = new FundCountExample();
        FundCountExample.Criteria criteria = example.createCriteria();

        if (req == null) {
            example.setOrderByClause("ret_1y desc");
        } else {
            if (CollectionUtils.isNotEmpty(req.getStrategys())) {
                criteria.andSubstrategyIn(req.getStrategys()
                        .stream()
                        .map(Integer::valueOf)
                        .collect(Collectors.toList()));
            }
            if (req.getRet1y() != null) {
                criteria.andRet1yBm1Between(new BigDecimal(req.getRet1y().getBegin()), new BigDecimal(req.getRet1y().getEnd()));
            }
            if (req.getMaxdrawdown1y() != null) {
                criteria.andMaxdrawdown1yBetween(new BigDecimal(req.getMaxdrawdown1y().getBegin()), new BigDecimal(req.getMaxdrawdown1y().getEnd()));
            }
            if (req.getStddev1y() != null) {
                criteria.andStddev1yBetween(new BigDecimal(req.getStddev1y().getBegin()), new BigDecimal(req.getStddev1y().getEnd()));
            }
            if (req.getSharperatio1y() != null) {
                criteria.andSharperatio1yBetween(new BigDecimal(req.getSharperatio1y().getBegin()), new BigDecimal(req.getSharperatio1y().getEnd()));
            }

            if (StringUtils.isNotEmpty(req.getRank())) {
                String sort = null;
                switch (req.getRank()) {
                    case "day":
                        sort = "ret_1day ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "week":
                        sort = "ret_1w ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "month":
                        sort = "ret_1m ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "season":
                        sort = "ret_3m ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "halfAYear":
                        sort = "ret_6m ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "year":
                        sort = "ret_1y ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "twoYear":
                        sort = "ret_2y ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "threeYear":
                        sort = "ret_3y ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "fiveYear":
                        sort = "ret_5y ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                    case "retIncep":
                        sort = "ret_incep ";
                        if (StringUtils.isNotEmpty(req.getRankSort())) {
                            sort = sort + req.getNetSort();
                        } else {
                            sort = sort + "desc,";
                        }
                        break;
                }

                sort = sort + "net_nav ";
                if (StringUtils.isNotEmpty(req.getNetSort())) {
                    sort = sort + req.getNetSort();
                } else {
                    sort = sort + "desc";
                }
                example.setOrderByClause(sort);
            } else {
                example.setOrderByClause("ret_1y desc");
            }
        }

        criteria.andDeleteTagEqualTo(0);
        return example;
    }

    @Override
    public Long getFundRankCountInfo(FundRankReq req) {
        FundCountExample example = filterFundCountReq(req);
        return this.fundCountMapper.countByExample(example);
    }

    private CommonResp<List<FundNavMessage>> getListCommonResp(Map<Long, IfaImportedFundNav> ifaNavMap,
                                                               List<FundNavMessage> resultMessage,
                                                               AtomicBoolean errorStatus,
                                                               List<List<Object>> read) {
        List<FundNavMessage> newResultMessage = handlerNetImportCheck(ifaNavMap, resultMessage, errorStatus, read);
        if (errorStatus.get()) {
            this.ifaImportedFundNavMapper.batchInsert(resultMessage.stream().map(ls -> {
                IfaImportedFundNav nav = new IfaImportedFundNav();
                BeanUtils.copyProperties(ls, nav);
                return nav;
            }).collect(Collectors.toList()));
            return CommonResp.success();
        }

        newResultMessage.sort(Comparator.comparing(FundNavMessage::getPriceDate).reversed());
        return CommonResp.error("9999", "数据错误,请您重新核对", newResultMessage);
    }

    private static final BigDecimal compare02 = new BigDecimal("0.2");

    @Override
    public CommonResp<List<FundNavMessage>> importNet(MultipartFile file, String fundId) {

        FundInfo fundInfo = this.fundInfoMapper.selectByPrimaryKey(fundId);
        if (fundInfo == null) {
            return CommonResp.error("获取不到基金信息");
        }

        Map<Long, IfaImportedFundNav> ifaNavMap = getIfaNavMap(fundId);

        try (InputStream inputStream = file.getInputStream()) {
            ExcelReader reader = ExcelUtil.getReader(inputStream);

            int size = reader.readAll().size();
            List<FundNavMessage> resultMessage = new ArrayList<>(size * 2);
            AtomicBoolean errorStatus = new AtomicBoolean(false);
            List<List<Object>> read = reader.read(2, size - 1);

            return getListCommonResp(ifaNavMap, resultMessage, errorStatus, read);
        } catch (IOException e) {
            return CommonResp.error("文件格式错误,无法读取，请您重新下载导入模板");
        }
    }

    @NotNull
    private Map<Long, IfaImportedFundNav> getIfaNavMap(String fundId) {
        IfaImportedFundNavExample example = new IfaImportedFundNavExample();
        example.createCriteria().andFundIdEqualTo(fundId);
        List<IfaImportedFundNav> ifaImportedFundNavs = this.ifaImportedFundNavMapper.selectByExample(example);

        Map<Long, IfaImportedFundNav> ifaNavMap;
        if (CollectionUtils.isEmpty(ifaImportedFundNavs)) {
            ifaNavMap = new HashMap<>(0);
        } else {
            ifaNavMap = ifaImportedFundNavs.stream().collect(Collectors.toMap(ls -> ls.getPriceDate().getTime(), ls -> ls, (newValue, oldValue) -> newValue));
        }
        return ifaNavMap;
    }

    private List<FundNavMessage> handlerNetImportCheck(Map<Long, IfaImportedFundNav> ifaNavMap, List<FundNavMessage> resultMessage, AtomicBoolean errorStatus, List<List<Object>> read) {
        checkParam(ifaNavMap, resultMessage, errorStatus, read);

        duplicateCheck(resultMessage, errorStatus);

        resultMessage.sort(Comparator.comparing(FundNavMessage::getPriceDate).reversed());

        return differCheck(resultMessage);
    }

    private void checkParam(Map<Long, IfaImportedFundNav> ifaNavMap, List<FundNavMessage> resultMessage, AtomicBoolean errorStatus, List<List<Object>> read) {
        read.forEach(ls -> {
            boolean errorRow = false;
            FundNavMessage message = new FundNavMessage();
            String row1 = (ls.get(0) + "").trim();
            if (NumberUtil.isNumber(row1)) {
                errorRow = true;
                message.setMessage("[日期格式错误]");
            } else {
                message.setPriceDate(DateUtil.date(Long.parseLong(row1)));
            }
            String row2 = (ls.get(1) + "").trim();
            if (!BigDecimalValidator.getInstance().isValid(row2)) {
                errorRow = true;
                if (StringUtils.isEmpty(message.getMessage())) {
                    message.setMessage("[单位净值为空或格式错误]");
                } else {
                    message.setMessage(message.getMessage() + ",[单位净值为空或格式错误]");
                }
            } else {
                message.setNav(new BigDecimal(row2));
            }

            String row3 = (ls.get(2) + "").trim();
            if (!BigDecimalValidator.getInstance().isValid(row3)) {
                errorRow = true;
                if (StringUtils.isEmpty(message.getMessage())) {
                    message.setMessage("[累计分红再投净值为空或格式错误]");
                } else {
                    message.setMessage(message.getMessage() + ",[累计分红再投净值为空或格式错误]");
                }
            } else {
                message.setCumulativeNav(new BigDecimal(row3));
            }
            if (errorRow) {
                message.setConfirmStatus(3);
                errorStatus.set(true);
            } else if (StringUtils.isNotEmpty(row1) && ifaNavMap.containsKey(Long.parseLong(row1))) {
                message.setConfirmStatus(3);
                errorStatus.set(true);
                if (StringUtils.isEmpty(message.getMessage())) {
                    message.setMessage("[相同日期存在重复数据]");
                } else {
                    message.setMessage(message.getMessage() + ",[累计分红再投净值为空或格式错误]");
                }
                FundNavMessage messageCopy = new FundNavMessage();
                BeanUtils.copyProperties(ifaNavMap.get(Long.parseLong(row1)), messageCopy);
                messageCopy.setConfirmStatus(3);
                resultMessage.add(messageCopy);
            }

            resultMessage.add(message);

        });
    }

    private List<FundNavMessage> differCheck(List<FundNavMessage> resultMessage) {
        int length = resultMessage.size();
        List<FundNavMessage> newResultMessage = new ArrayList<>(length);
        for (int i = 0; i < length; i++) {
            FundNavMessage message = resultMessage.get(i);
            if (message.getConfirmStatus() == 1 && (i + 1) < length) {
                FundNavMessage messageAdd = resultMessage.get(i + 1);
                if (resultMessage.get(i + 1).getConfirmStatus() == 1) {

                    BigDecimal navSubtract = subtractAbs(message.getNav(), messageAdd.getNav());
                    if (compare02.compareTo(navSubtract) < 0) {
                        message.setConfirmStatus(2);
                        message.setMessage("[单位净值]与上次对比相差超过20%");
                    }

                    BigDecimal cumSubtract = subtractAbs(message.getCumulativeNav(), messageAdd.getCumulativeNav());
                    if (compare02.compareTo(cumSubtract) < 0) {
                        message.setConfirmStatus(2);
                        if (StringUtils.isNotEmpty(message.getMessage())) {
                            message.setMessage(",[累计分红再投净值]与上次对比相差超过20%");
                        } else {
                            message.setMessage("[累计分红再投净值]与上次对比相差超过20%");
                        }
                    }
                } else {
                    i++;
                }
                newResultMessage.add(message);
            }
            newResultMessage.add(message);
        }
        return newResultMessage;
    }

    private void duplicateCheck(List<FundNavMessage> resultMessage, AtomicBoolean errorStatus) {
        Map<Long, Long> priceDateCountMap = resultMessage
                .stream()
                .filter(message -> !message.getConfirmStatus().equals(3))
                .collect(Collectors.groupingBy(ls -> ls.getPriceDate().getTime(), Collectors.counting()));

        Set<Long> duplicateSet = new HashSet<>(priceDateCountMap.size() >> 1);
        priceDateCountMap.forEach((key, value) -> {
            if (value > 1) {
                duplicateSet.add(key);
            }
        });

        resultMessage.forEach(message -> {
            Date priceDate = message.getPriceDate();
            if (priceDate != null && duplicateSet.contains(priceDate.getTime())) {
                errorStatus.set(true);
                message.setConfirmStatus(3);
                if (StringUtils.isEmpty(message.getMessage())) {
                    message.setMessage("[相同日期存在重复数据]");
                } else {
                    message.setMessage(message.getMessage() + ",[累计分红再投净值为空或格式错误]");
                }
            }
        });
    }

    private ArrayList<TrackRecordVO> getPrivateFundTrackRecordVOS(IfaImportedFundCount fundCount) {
        return Lists.newArrayList(
                new TrackRecordVO("近三个月", multiply100(fundCount.getRet3m()), multiply100(fundCount.getRet3mBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近半年", multiply100(fundCount.getRet6m()), multiply100(fundCount.getRet6mBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近一年", multiply100(fundCount.getRet1y()), multiply100(fundCount.getRet1yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近二年", multiply100(fundCount.getRet2y()), multiply100(fundCount.getRet2yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近三年", multiply100(fundCount.getRet3y()), multiply100(fundCount.getRet3yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近四年", multiply100(fundCount.getRet4y()), multiply100(fundCount.getRet4yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("近五年", multiply100(fundCount.getRet5y()), multiply100(fundCount.getRet5yBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()),
                new TrackRecordVO("成立以来", multiply100(fundCount.getRetIncep()), multiply100(fundCount.getRetIncepBm1()), fundCount.getEndDate() == null ? null : fundCount.getEndDate().getTime()));

    }

}
